/*
 * Copyright (C) 2016 Fairphone B.V.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.fairphone.hiccup.app;

import android.app.AlarmManager;
import android.app.IntentService;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.preference.PreferenceManager;

import com.fairphone.hiccup.app.R;

import java.util.Date;

public class HiccupService extends IntentService {
    static final String ACTION_REBOOT_COMPLETE = "com.fairphone.hiccup.app.action.REBOOT_COMPLETED";
    static final String ACTION_SHUTDOWN        = "com.fairphone.hiccup.app.action.SHUTDOWN";
    static final String ACTION_CHECK_UPLOAD    = "com.fairphone.hiccup.app.action.CHECK_UPLOAD";
    static final String ACTION_GENERATE_FAKE_REPORT = "com.fairphone.hiccup.app.action.SIMULATE_CRASH";
    static final String ACTION_CHECK_HEARTBEAT  = "com.fairphone.hiccup.app.action.CHECK_PING_BACK";

    static private final long HEARTBEAT_INTERVAL_MS = 24*60*60*1000;

    public HiccupService() {
        super("HiccupService");
    }

    @Override
    protected void onHandleIntent(Intent intent) {
        if (intent != null) {
            final String action = intent.getAction();
            if (ACTION_REBOOT_COMPLETE.equals(action)) {
                handleRebootComplete();
            }
            if (ACTION_SHUTDOWN.equals(action)) {
                handleShutdown();
            }
            if (ACTION_CHECK_UPLOAD.equals(action)) {
                checkUpload();
            }
            if (ACTION_GENERATE_FAKE_REPORT.equals(action)) {
                generateFakeCrashReport();
            }
            if (ACTION_CHECK_HEARTBEAT.equals(action)) {
                checkAndSetHeartbeatTimer();
            }
        }
    }

    private void generateFakeCrashReport() {
        HiccupReport.createFakeCrashReport(this);
    }

    private void checkUpload() {
        HiccupUploader.trigger(this);
    }

    private boolean wasProperlyShutdown() {
        SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(this);
        return preferences.getBoolean(getString(R.string.PREF_KEY_WAS_PROPERLY_SHUTDOWN), true);
    }

    private void handleRebootComplete() {
        detectAndRegisterCrash();
        checkAndSetHeartbeatTimer();
        prepareCrashDetection();
    }

    private void checkAndSetHeartbeatTimer() {
        Date lastHeartbeatDate = HiccupSettings.getLastHeartbeatDate(this);
        Date now = new Date();
        if (now.getTime() - lastHeartbeatDate.getTime() > HEARTBEAT_INTERVAL_MS) {
            HiccupReport.createHeartbeatReport(this);
            HiccupSettings.setLastHeartbeatDate(this, now);
            lastHeartbeatDate = now;
        }
        setHeartbeatTimer(lastHeartbeatDate);
    }

    private void setHeartbeatTimer(Date lastHeartbeatDate) {
        Intent intent = new Intent(this, HiccupService.class).setAction(ACTION_CHECK_HEARTBEAT);
        PendingIntent pendingIntent = PendingIntent.getService(
                this, 0, intent, PendingIntent.FLAG_UPDATE_CURRENT);
        AlarmManager alarmManager = (AlarmManager) this.getSystemService(Context.ALARM_SERVICE);
        alarmManager.setInexactRepeating(AlarmManager.ELAPSED_REALTIME,
                lastHeartbeatDate.getTime()+ HEARTBEAT_INTERVAL_MS,
                AlarmManager.INTERVAL_DAY, pendingIntent);
    }

    private void detectAndRegisterCrash() {
        HiccupUtil.LOG("Reboot complete: Checking reboot state.");
        if (!wasProperlyShutdown()
                && HiccupSettings.isCrashReportingEnabled(this)
                && HiccupUtil.isHiccupdRunning()) {
            HiccupUtil.LOG("Reboot complete: Crash detected.");
            HiccupReport.createCrashReport(this);
        } else {
            HiccupUtil.LOG("Reboot complete: No Crash detected.");
        }
    }

    private void prepareCrashDetection() {
        SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(this);
        SharedPreferences.Editor editor = preferences.edit();
        editor.putBoolean(getString(R.string.PREF_KEY_WAS_PROPERLY_SHUTDOWN), false);
        editor.apply();
    }

    private void handleShutdown() {
        SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(this);
        SharedPreferences.Editor editor = preferences.edit();
        editor.putBoolean(getString(R.string.PREF_KEY_WAS_PROPERLY_SHUTDOWN), true);
        editor.apply();
    }
}